/*
 * hptoken.h  -- An HP-2000 Timeshared BASIC System tape manipulation program.
 *
 * Copyright (c) 2005, Scott G. Taylor
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * SCOTT G. TAYLOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Scott G. Taylor shall not
 * be used in advertising or otherwise to promote the sale, use or other dealings
 * in this Software without prior written authorization from Scott G. Taylor.
 */

/*
 *	The table below gives the internal representation of the BASIC
 *	operators.  Those operators which manipulate the formula
 *	evaluation stack during execution have associate priorities.
 *	BASIC statement types are represented in the following table.
 *	They can be differentiated from BASIC operators because statement
 *	types must appear only as the first operator in a statement.  All
 *	numbers are in octal notation.
 *
 */

struct BASIC_Tokens Bop_ACCESS[] = {
	{ 0,	0,	"" 		},	/* end-of-formula */
	{ 1,	0,	"\""		},
	{ 2,	0,	","		},
	{ 3,	0,	";"		},	/* Listed incorrectly as (:) in the Access IMS */
	{ 4,	0,	"#"		},	/* FILE */
	{ 5,	0,	"(op 5)"	},
	{ 6,	0,	"(op 6)"	},
	{ 7,	0,	"(op 7)"	},
	{ 010,	1,	")"		},
	{ 011,	1,	"]"		},
	{ 012,	013,	"["		},
	{ 013,	011,	"("		},
	{ 014,	011,	"+"		},	/* unary */
	{ 015,	011,	"-"		},	/* unary */
	{ 016,	2,	","		},	/* subscript */
	{ 017,	2,	"="		},	/* assignment */
	{ 020,	7,	"+"		},
	{ 021,	7,	"-"		},
	{ 022,	010,	"*"		},
	{ 023,	010,	"/"		},
	{ 024,	012,	"^"		},
	{ 025,	5,	">"		},
	{ 026,	5,	"<"		},
	{ 027,	5,	"#"		},
	{ 030,	5,	"="		},	/* equal */
	{ 031,	4,	"(op 031)"	},
	{ 032,	4,	" AND "		},
	{ 033,	3,	" OR "		},
	{ 034,	6,	" MIN "		},
	{ 035,	6,	" MAX "		},
	{ 036,	5,	" <> "		},
	{ 037,	5,	" >= ",		},
	{ 040,	5,	" <= ",		},
	{ 041,	011,	" NOT "		},
	{ 042,	012,	"**"		},
	{ 043,	0,	" USING "	},
	{ 044,	0,	" RR "		},
	{ 045,	0,	" WR "		},
	{ 046,	0,	" NR "		},
	{ 047,	0,	" ERROR "	},
	{ 050,	0,	"(op 050)"	},
	{ 051,	0,	"(op 051)"	},
	{ 052,	0,	"(op 052)"	},
	{ 053,	0,	"(op 053)"	},
	{ 054,	0,	"(op 054)"	},
	{ 055,	0,	"(op 055)"	},
	{ 056,	0,	"(op 056)"	},
	{ 057,	0,	"(op 057)"	},
	{ 060,	0,	" END "		},
	{ 061,	0,	"(op 061)"	},
	{ 062,	0,	"(op 062)"	},
	{ 063,	0,	" INPUT "	},
	{ 064,	0,	" READ "	},
	{ 065,	0,	" PRINT "	},
	{ 066,	0,	"(op 066)"	},
	{ 067,	0,	"(op 067)"	},
	{ 070,	0,	"(op 070)"	},
	{ 071,	0,	"(op 071)"	},
	{ 072,	0,	"(op 072)"	},
	{ 073,	0,	"(op 073)"	},
	{ 074,	0,	" OF "		},
	{ 075,	0,	" THEN "	},
	{ 076,	0,	" TO "		},
	{ 077,	0,	" STEP "	}
};

struct BASIC_Tokens Bop_2000F[] = {
	{ 0,	0,	"" 		},	/* end-of-formula */
	{ 1,	0,	"\""		},
	{ 2,	0,	","		},
	{ 3,	0,	";"		},
	{ 4,	0,	"#"		},	/* FILE */
	{ 5,	0,	"(op 5)"	},
	{ 6,	0,	"(op 6)"	},
	{ 7,	0,	"(op 7)"	},
	{ 010,	1,	")"		},
	{ 011,	1,	"]"		},
	{ 012,	013,	"["		},
	{ 013,	011,	"("		},
	{ 014,	011,	"+"		},	/* unary */
	{ 015,	011,	"-"		},	/* unary */
	{ 016,	2,	","		},	/* subscript */
	{ 017,	2,	"="		},	/* assignment */
	{ 020,	7,	"+"		},
	{ 021,	7,	"-"		},
	{ 022,	010,	"*"		},
	{ 023,	010,	"/"		},
	{ 024,	012,	"^"		},
	{ 025,	5,	">"		},
	{ 026,	5,	"<"		},
	{ 027,	5,	"#"		},
	{ 030,	5,	"="		},	/* equal */
	{ 031,	4,	"(op 031)"	},
	{ 032,	4,	" AND "		},
	{ 033,	3,	" OR "		},
	{ 034,	6,	" MIN "		},
	{ 035,	6,	" MAX "		},
	{ 036,	5,	" <> "		},
	{ 037,	5,	" >= ",		},
	{ 040,	5,	" <= ",		},
	{ 041,	011,	" NOT "		},
	{ 042,	012,	" ASSIGN "	},
	{ 043,	0,	" USING "	},
	{ 044,	0,	" IMAGE "	},
	{ 045,	0,	" COM "		},
	{ 046,	0,	" LET "		},
	{ 047,	0,	" DIM "		},
	{ 050,	0,	" DEF "		},
	{ 051,	0,	" REM "		},
	{ 052,	0,	" GOTO "	},
	{ 053,	0,	" IF "		},
	{ 054,	0,	" FOR "		},
	{ 055,	0,	" NEXT "	},
	{ 056,	0,	" GOSUB "	},
	{ 057,	0,	" RETURN"	},
	{ 060,	0,	" END "		},
	{ 061,	0,	" STOP"		},
	{ 062,	0,	" DATA "	},
	{ 063,	0,	" INPUT "	},
	{ 064,	0,	" READ "	},
	{ 065,	0,	" PRINT "	},
	{ 066,	0,	" RESTORE "	},
	{ 067,	0,	" MAT "		},
	{ 070,	0,	" FILES "	},
	{ 071,	0,	" CHAIN "	},
	{ 072,	0,	" ENTER "	},
	{ 073,	0,	""		}, /* Implied LET */
	{ 074,	0,	" OF "		},
	{ 075,	0,	" THEN "	},
	{ 076,	0,	" TO "		},
	{ 077,	0,	" STEP "	}
};

struct BASIC_Tokens Bop_2000E[] = {
	{ 0,	0,	"" 		},	/* end-of-formula */
	{ 1,	0,	"\""		},
	{ 2,	0,	","		},
	{ 3,	0,	";"		},
	{ 4,	0,	"#"		},	/* FILE */
	{ 5,	0,	"(op 5)"	},
	{ 6,	0,	"(op 6)"	},
	{ 7,	0,	"(op 7)"	},
	{ 010,	1,	")"		},
	{ 011,	1,	"]"		},
	{ 012,	013,	"["		},
	{ 013,	013,	"("		},
	{ 014,	011,	"+"		},	/* unary */
	{ 015,	011,	"-"		},	/* unary */
	{ 016,	2,	","		},	/* subscript */
	{ 017,	2,	"="		},	/* assignment */
	{ 020,	7,	"+"		},
	{ 021,	7,	"-"		},
	{ 022,	010,	"*"		},
	{ 023,	010,	"/"		},
	{ 024,	012,	"^"		},
	{ 025,	5,	">"		},
	{ 026,	5,	"<"		},
	{ 027,	5,	"#"		},
	{ 030,	5,	"="		},	/* equal */
	{ 031,	0,	"(op 031)"	},
	{ 032,	0,	"(op 031)"	},
	{ 033,	4,	" AND "		},
	{ 034,	3,	" OR "		},
	{ 035,	6,	" MIN "		},
	{ 036,	6,	" MAX "		},
	{ 037,	5,	" <> "		},
	{ 040,	5,	" >= ",		},
	{ 041,	5,	" <= ",		},
	{ 042,	011,	" NOT "		},
	{ 043,	0,	"(op 043)"	},
	{ 044,	0,	"(op 044)"	},
	{ 045,	0,	" COM "		},
	{ 046,	0,	" LET "		},
	{ 047,	0,	" DIM "		},
	{ 050,	0,	" DEF "		},
	{ 051,	0,	" REM "		},
	{ 052,	0,	" GOTO "	},
	{ 053,	0,	" IF "		},
	{ 054,	0,	" FOR "		},
	{ 055,	0,	" NEXT "	},
	{ 056,	0,	" GOSUB "	},
	{ 057,	0,	" RETURN"	},
	{ 060,	0,	" END "		},
	{ 061,	0,	" STOP"		},
	{ 062,	0,	" DATA "	},
	{ 063,	0,	" INPUT "	},
	{ 064,	0,	" READ "	},
	{ 065,	0,	" PRINT "	},
	{ 066,	0,	" RESTORE "	},
	{ 067,	0,	" MAT "		},
	{ 070,	0,	" FILES "	},
	{ 071,	0,	" CHAIN "	},
	{ 072,	0,	" "		}, /* IMS says "ENTER", but it should be implied LET. */
	{ 073,	0,	"(op 073)"	}, /* IMS says implied LET, but use is unknown.	*/
	{ 074,	0,	" OF "		},
	{ 075,	0,	" THEN "	},
	{ 076,	0,	" TO "		},
	{ 077,	0,	" STEP "	}
};

struct BASIC_Tokens Bop_2000A[] = {
	{ 0,	0,	"" 		},	/* end-of-formula */
	{ 1,	0,	"\""		},
	{ 2,	0,	","		},
	{ 3,	0,	";"		},
	{ 4,	0,	"#"		},	/* FILE */
	{ 5,	0,	"(op 5)"	},
	{ 6,	0,	"(op 6)"	},
	{ 7,	0,	"(op 7)"	},
	{ 010,	1,	")"		},
	{ 011,	1,	"]"		},
	{ 012,	013,	"["		},
	{ 013,	013,	"("		},
	{ 014,	011,	"+"		},	/* unary */
	{ 015,	011,	"-"		},	/* unary */
	{ 016,	2,	","		},	/* subscript */
	{ 017,	2,	"="		},	/* assignment */
	{ 020,	7,	"+"		},
	{ 021,	7,	"-"		},
	{ 022,	010,	"*"		},
	{ 023,	010,	"/"		},
	{ 024,	012,	"^"		},
	{ 025,	5,	">"		},
	{ 026,	5,	"<"		},
	{ 027,	5,	"#"		},
	{ 030,	5,	"="		},	/* equal */
	{ 031,	0,	"(op 031)"	},
	{ 032,	0,	"(op 032)"	},
	{ 033,	0,	"(op 033)"	},
	{ 034,	0,	"(op 034)"	},
	{ 035,	0,	"(op 035)"	},
	{ 036,	4,	" AND "		},
	{ 037,	3,	" OR "		},
	{ 040,	6,	" MIN "		},
	{ 041,	6,	" MAX "		},
	{ 042,	5,	" <> "		},
	{ 043,	5,	" >= ",		},
	{ 044,	5,	" <= ",		},
	{ 045,	011,	" NOT "		},
	{ 046,	0,	" LET "		},
	{ 047,	0,	" DIM "		},
	{ 050,	0,	" DEF "		},
	{ 051,	0,	" REM "		},
	{ 052,	0,	" GOTO "	},
	{ 053,	0,	" IF "		},
	{ 054,	0,	" FOR "		},
	{ 055,	0,	" NEXT "	},
	{ 056,	0,	" GOSUB "	},
	{ 057,	0,	" RETURN"	},
	{ 060,	0,	" END "		},
	{ 061,	0,	" STOP"		},
	{ 062,	0,	" DATA "	},
	{ 063,	0,	" INPUT "	},
	{ 064,	0,	" READ "	},
	{ 065,	0,	" PRINT "	},
	{ 066,	0,	" RESTORE "	},
	{ 067,	0,	" MAT "		},
	{ 070,	0,	" FILES "	},
	{ 071,	0,	""		}, /* Implied LET */
	{ 072,	0,	"(op 072)"	},
	{ 073,	0,	"(op 073)"	},
	{ 074,	0,	" OF "		},
	{ 075,	0,	" THEN "	},
	{ 076,	0,	" TO "		},
	{ 077,	0,	" STEP "	}
};

struct BASIC_Tokens Bst_ACCESS[] = {
	{ 0,	0,	"(st 000)"	},
	{ 1,	0,	"(st 001)"	},
	{ 2,	0,	"(st 002)"	},
	{ 3,	0,	"(st 003)"	},
	{ 4,	0,	"(st 004)"	},
	{ 5,	0,	"(st 005)"	},
	{ 6,	0,	"(st 006)"	},
	{ 7,	0,	"(st 007)"	},
	{ 010,	0,	"(st 010)"	},
	{ 011,	0,	"(st 011)"	},
	{ 012,	0,	"(st 012)"	},
	{ 013,	0,	"(st 013)"	},
	{ 014,	0,	"(st 014)"	},
	{ 015,	0,	"(st 015)"	},
	{ 016,	0,	"(st 016)"	},
	{ 017,	0,	"(st 017)"	},
	{ 020,	0,	"(st 020)"	},
	{ 021,	0,	"(st 021)"	},
	{ 022,	0,	"(st 022)"	},
	{ 023,	0,	"(st 023)"	},
	{ 024,	0,	"(st 024)"	},
	{ 025,	0,	"(st 025)"	},
	{ 026,	0,	"(st 026)"	},
	{ 027,	0,	"(st 027)"	},
	{ 030,	0,	"(st 030)"	},
	{ 031,	0,	"(st 031)"	},
	{ 032,	0,	"SYSTEM"	},
	{ 033,	0,	"CONVERT"	},
	{ 034,	0,	"LOCK"		},
	{ 035,	0,	"UNLOCK"	},
	{ 036,	0,	"CREATE"	},
	{ 037,	0,	"PURGE"		},
	{ 040,	0,	"ADVANCE"	},
	{ 041,	0,	"UPDATE"	},
	{ 042,	0,	"ASSIGN"	},
	{ 043,	0,	"LINPUT"	},
	{ 044,	0,	"IMAGE"		},
	{ 045,	0,	"COM"		},
	{ 046,	0,	"LET"		},
	{ 047,	0,	"DIM"		},
	{ 050,	0,	"DEF"		},
	{ 051,	0,	"REM"		},
	{ 052,	0,	"GOTO"		},
	{ 053,	0,	"IF"		},
	{ 054,	0,	"FOR"		},
	{ 055,	0,	"NEXT"		},
	{ 056,	0,	"GOSUB"		},
	{ 057,	0,	"RETURN"	},
	{ 060,	0,	"END"		},
	{ 061,	0,	"STOP"		},
	{ 062,	0,	"DATA"		},
	{ 063,	0,	"INPUT"		},
	{ 064,	0,	"READ"		},
	{ 065,	0,	"PRINT"		},
	{ 066,	0,	"RESTORE"	},
	{ 067,	0,	"MAT"		},
	{ 070,	0,	"FILES"		},
	{ 071,	0,	"CHAIN"		},
	{ 072,	0,	"ENTER"		},
	{ 073,	0,	""		},	/* Implied LET */
	{ 074,	0,	"(st 074)"	},
	{ 075,	0,	"(st 075)"	},
	{ 076,	0,	"(st 076)"	},
	{ 077,	0,	"(st 077)"	}
};

struct BASIC_Tokens Bfunc[] = {
/*
 *	     /--------------------- 
 *	     |/-------------------- 
 *	     ||/------------------- ARGS: 1 or more
 *           |||/------------------ Returns STRING, else NUMERIC
 *	     ||||
 *	     vvvv
 */
	{ 0,	0,	"CTL"	},
	{ 1,	0,	"TAB"	},
	{ 2,	0,	"LIN"	},
	{ 3,	0,	"SPA"	},
	{ 4,	1,	"TAN"	},
	{ 5,	1,	"ATN"	},
	{ 6,	1,	"EXP"	},
	{ 7,	1,	"LOG"	},
	{ 010,	1,	"ABS"	},
	{ 011,	1,	"SQR"	},
	{ 012,	1,	"INT"	},
	{ 013,	1,	"RND"	},
	{ 014,	1,	"SGN"	},
	{ 015,	1,	"LEN"	},
	{ 016,	1,	"TYP"	},
	{ 017,	1,	"TIM"	},
	{ 020,	1,	"SIN"	},
	{ 021,	1,	"COS"	},
	{ 022,	1,	"BRK"	},
	{ 023,	1,	"ITM"	},
	{ 024,	1,	"REC"	},
	{ 025,	0x01,	"NUM"	},
	{ 026,	0x02,	"POS"	},
	{ 027,	0x03,	"CHR$"	},
	{ 030,	1,	"UPS$"	},
	{ 031,	1,	"SYS"	},
	{ 032,	1,	"(F032)"},
	{ 033,	5,	"ZER"	},
	{ 034,	5,	"CON"	},
	{ 035,	5,	"IDN"	},
	{ 036,	6,	"INV"	},
	{ 037,	6,	"TRN"	}
};
